#!/usr/bin/env python3

import csv
import os
import string
import sys

# Get cargos from cargotable
with open(os.path.join(sys.path[0], 'cargotable.pnml')) as f:
    f_str = f.read()
    cargo_begin = f_str.index('{') + 1
    cargo_end = f_str.index('}')
    cargos = {i.strip() for i in f_str[cargo_begin:cargo_end].split(',')}

def check_properties(props_in):
    """Apply props to template, overwrite with props from csv if possible."""
    default_props = {
        "intr_y": 1920,
        "model_life": "VEHICLE_NEVER_EXPIRES",
        "vehicle_life": 255,
        "refittable_cargo_classes": 0,
        "non_refittable_cargo_classes": 0,
        "cargo_allow_refit": "",
        "cargo_disallow_refit": "",
        "loading_speed": 0,
        "cost_factor": 0,
        "running_cost_factor": 0,

        "speed": 0,
        "power": 0,
        "capacity": 0,
        "weight": 1,
        "engine_class": "ENGINE_CLASS_STEAM",
        "te": 100
    }
    props_out = props_in
    for prop in default_props:
        if prop not in props_out or not props_out[prop]:
            props_out[prop] = default_props[prop]
    # Make disallowed cargos (all cargos - allowed)
    cargo_disallow = cargos
    cargo_allow = \
        {i.strip() for i in props_out["cargo_allow_refit"].split(',')}
    props_out["cargo_disallow_refit"] = \
        ', '.join(cargo_disallow - cargo_allow)

    return props_out

train_props_template = """
item(FEAT_TRAINS, item_$name) {
    property {
        name: string(str_name_$name);
        climates_available: ALL_CLIMATES;
        introduction_date: date($intr_y,01,01);
        model_life: $model_life;
        retire_early: 0;
        vehicle_life: $vehicle_life;
        reliability_decay: 20;
        refittable_cargo_classes: $refittable_cargo_classes;
        non_refittable_cargo_classes: $non_refittable_cargo_classes;
        cargo_allow_refit: [$cargo_allow_refit];
        cargo_disallow_refit: [$cargo_disallow_refit];
        loading_speed: 0;
        cost_factor: $cost_factor;
        running_cost_factor: $running_cost_factor;
        cargo_age_period: 185;

        sprite_id: SPRITE_ID_NEW_TRAIN;
        speed: $speed km/h;
        misc_flags: bitmask(TRAIN_FLAG_2CC, TRAIN_FLAG_FLIP);
        refit_cost: 0;
        track_type: RAIL;
        ai_special_flag: 0;
        power: $power hpM;
        running_cost_base: RUNNING_COST_DIESEL;
        dual_headed: 0;
        default_cargo_type: 0;
        cargo_capacity: $capacity;
        weight: $weight ton;
        engine_class: $engine_class;
        extra_power_per_wagon: 0;
        tractive_effort_coefficient: $te.0 / 255;
        air_drag_coefficient: 0;
        length: 8;
        visual_effect_and_powered: visual_effect_and_powered(
            VISUAL_EFFECT_DISABLE, 0, DISABLE_WAGON_POWER);
        extra_weight_per_wagon: 0;
        bitmask_vehicle_info: 0;
    }
}
"""

build_dir = sys.argv[1]
src_paths = sys.argv[2:]

temp = string.Template(train_props_template)
prop_list = []
lang_list = []
for file_ in src_paths:
    with open(file_) as f:
        reader = csv.DictReader(f)
        for veh in reader:
            # Ignore rows with empty name cell
            if not veh["name"]:
                continue
            else:
                props = check_properties(veh)
                prop_list.append(temp.substitute(props))
                lang_list.append("str_name_{} :{}\n".format(
                    props["name"], props["long_name"]))
props_filename = "train_props.pnml"
lang_filename = "train_strings.plang"
with open(os.path.join(build_dir, props_filename), 'w') as p:
    p.write(''.join(prop_list))
with open(os.path.join(build_dir, lang_filename), 'w') as l:
    l.write(''.join(lang_list))
